@@ -1,3 +1,4 @@
+import enum
 from .openapiartplugin import OpenApiArtPlugin
 import os
 import subprocess
@@ -155,6 +156,7 @@ class OpenApiArtGo(OpenApiArtPlugin):
         super(OpenApiArtGo, self).__init__(**kwargs)
         self._api = FluentStructure()
         self._api_interface_methods = []
+        self._base_url = ""
         self._oapi_go_types = {
             "string": "string",
             "boolean": "bool",
@@ -171,15 +173,28 @@ class OpenApiArtGo(OpenApiArtPlugin):
         }
 
     def generate(self, openapi):
+        self._base_url = ""
         self._openapi = openapi
         self._ux_path = os.path.normpath(os.path.join(self._output_dir, "..", os.path.split(self._go_sdk_package_dir)[-1]))
         self._protoc_path = os.path.normpath(os.path.join(self._ux_path, self._protobuf_package_name))
         self._structs = {}
+        self._get_base_url()
         self._write_mod_file()
         self._write_go_file()
         self._format_go_file()
         self._tidy_mod_file()
 
+    def _get_base_url(self):
+        self._base_url = ""
+        if "servers" in self._openapi:
+            server = self._openapi["servers"][0]
+            try:
+                self._base_url = server['variables']['basePath']['default']
+                if not self._base_url.startswith("/"):
+                    self._base_url = "/" + self._base_url
+            except KeyError:
+                pass
+
     def _write_mod_file(self):
         self._filename = os.path.normpath(os.path.join(self._ux_path, "go.mod"))
         self.default_indent = "    "
@@ -227,7 +242,7 @@ class OpenApiArtGo(OpenApiArtPlugin):
         self._write('import "google.golang.org/grpc"')
         self._write('import "github.com/ghodss/yaml"')
         self._write('import "google.golang.org/protobuf/encoding/protojson"')
-        self._write('import "github.com/golang/protobuf/proto"')
+        self._write('import "google.golang.org/protobuf/proto"')
         go_pkg_fp = self._fp
         go_pkg_filename = self._filename
         self._filename = os.path.normpath(os.path.join(self._ux_path, "common.go"))
@@ -310,6 +325,9 @@ class OpenApiArtGo(OpenApiArtPlugin):
             external_name=self._get_external_struct_name(self._go_sdk_package_name)
         )
         for url, path_object in self._openapi["paths"].items():
+            http_url = self._base_url + url
+            if http_url.startswith("/"):
+                http_url = http_url[1:]
             for operation_id in self._get_parser("$..operationId").find(path_object):
                 path_item_object = operation_id.context.value
                 rpc = FluentRpc()
@@ -358,7 +376,7 @@ class OpenApiArtGo(OpenApiArtPlugin):
                         interface=new.interface,
                         struct=new.struct,
                     )
-                    rpc.description = "// {} {}".format(rpc.operation_name, rpc.description.lstrip("// ")) 
+                    rpc.description = "// {} {}".format(rpc.operation_name, rpc.description.lstrip("// "))
                     # """
                     #     // Performs {operation_name} on user provided {interface} and returns {request_return_type}
                     #     // or returns error on failure
@@ -390,9 +408,12 @@ class OpenApiArtGo(OpenApiArtPlugin):
                     )
                     if url.startswith("/"):
                         url = url[1:]
-                    http.request = """api.httpSendRecv("{url}", {struct}.ToJson(), "{method}")""".format(
+                    http.request = """{struct}Json, err := {struct}.ToJson()
+                    if err != nil {{return nil, err}}
+                    resp, err := api.httpSendRecv("{url}", {struct}Json, "{method}")
+                    """.format(
                         operation_name=http.operation_name,
-                        url=url,
+                        url=http_url,
                         struct=new.struct,
                         method=str(operation_id.context.path.fields[0]).upper(),
                     )
@@ -415,8 +436,8 @@ class OpenApiArtGo(OpenApiArtPlugin):
                     rpc.http_call = """return api.http{operation_name}()""".format(
                         operation_name=rpc.operation_name,
                     )
-                    http.request = """api.httpSendRecv("{url}", "", "{method}")""".format(
-                        url=url, method=str(operation_id.context.path.fields[0]).upper()
+                    http.request = """resp, err := api.httpSendRecv("{url}", "", "{method}")""".format(
+                        url=http_url, method=str(operation_id.context.path.fields[0]).upper()
                     )
                     http.method = """http{rpc_method}""".format(rpc_method=rpc.method)
                 for ref in self._get_parser("$..responses").find(path_item_object):
@@ -451,11 +472,40 @@ class OpenApiArtGo(OpenApiArtPlugin):
             // grpcConnect builds up a grpc connection
             func (api *{internal_struct_name}) grpcConnect() error {{
                 if api.grpcClient == nil {{
-                    conn, err := grpc.Dial(api.grpc.location, grpc.WithInsecure())
+                    ctx, cancelFunc := context.WithTimeout(context.Background(), api.grpc.dialTimeout)
+                    defer cancelFunc()
+                    conn, err := grpc.DialContext(ctx, api.grpc.location, grpc.WithInsecure())
                     if err != nil {{
                         return err
                     }}
                     api.grpcClient = {pb_pkg_name}.NewOpenapiClient(conn)
+                    api.grpc.clientConnection = conn
+                }}
+                return nil
+            }}
+
+            func (api *{internal_struct_name}) grpcClose() error {{
+                if api.grpc != nil {{
+                    if api.grpc.clientConnection != nil {{
+                        err := api.grpc.clientConnection.Close()
+                        if err != nil {{
+                            return err
+                        }}
+                    }}
+                }}
+                api.grpcClient = nil
+                api.grpc = nil
+                return nil
+            }}
+
+            func (api *{internal_struct_name}) Close() error {{
+                if api.hasGrpcTransport() {{
+                    err := api.grpcClose()
+		            return err
+                }}
+                if api.hasHttpTransport() {{
+                    api.http = nil
+		            api.httpClient.client = nil
                 }}
                 return nil
             }}
@@ -526,7 +576,7 @@ class OpenApiArtGo(OpenApiArtPlugin):
                 external_interface_name=self._api.external_interface_name,
                 method_signatures=method_signatures,
                 description="// {} {}".format(
-                    self._api.external_interface_name, 
+                    self._api.external_interface_name,
                     self._get_description(self._openapi["info"], True).lstrip("// "))
             )
         )
@@ -638,7 +688,7 @@ class OpenApiArtGo(OpenApiArtPlugin):
                 )
             self._write(
                 """func (api *{internal_struct_name}) {method} {{
-                    resp, err := {request}
+                    {request}
                     if err != nil {{
                         return nil, err
                     }}
@@ -718,19 +768,21 @@ class OpenApiArtGo(OpenApiArtPlugin):
         internal_items_nil = []
         for field in new.interface_fields:
             if field.struct and field.isArray is False:
-                internal_items.append("{}Holder {}".format(
-                    field.name[0].lower() + field.name[1:], field.type
+                internal_items.append("{} {}".format(
+                    self._get_holder_name(field), field.type
                 ))
-                internal_items_nil.append("obj.{}Holder = nil".format(
-                    field.name[0].lower() + field.name[1:]))
+                internal_items_nil.append("obj.{} = nil".format(
+                    self._get_holder_name(field)))
             if field.adder_method is not None and field.isArray:
-                internal_items.append("{}Holder {}".format(
-                    field.name[0].lower() + field.name[1:], new.interface + field.external_struct + "Iter"
+                internal_items.append("{} {}".format(
+                    self._get_holder_name(field), new.interface + field.external_struct + "Iter"
                 ))
-                internal_items_nil.append("obj.{}Holder = nil".format(
-                    field.name[0].lower() + field.name[1:]))
+                internal_items_nil.append("obj.{} = nil".format(
+                    self._get_holder_name(field)))
         self._write(
-            """type {struct} struct {{
+            """
+            // ***** {interface} *****
+            type {struct} struct {{
                 obj *{pb_pkg_name}.{interface}
                 {internal_items}
             }}
@@ -746,20 +798,25 @@ class OpenApiArtGo(OpenApiArtPlugin):
             }}
 
             func (obj *{struct}) SetMsg(msg *{pb_pkg_name}.{interface}) {interface} {{
+                {nil_call}
                 proto.Merge(obj.obj, msg)
                 return obj
             }}
 
-            func (obj *{struct}) ToPbText() string {{
+            func (obj *{struct}) ToPbText() (string, error) {{
                 vErr := obj.Validate()
                 if vErr != nil {{
-                    panic(vErr)
+                    return "", vErr
                 }}
-                return proto.MarshalTextString(obj.Msg())
+                protoMarshal, err := proto.Marshal(obj.Msg())
+                if err != nil {{
+                    return "", err
+                }}
+                return string(protoMarshal), nil
             }}
 
             func (obj *{struct}) FromPbText(value string) error {{
-                retObj := proto.UnmarshalText(value, obj.Msg())
+                retObj := proto.Unmarshal([]byte(value), obj.Msg())
                 if retObj != nil {{
                     return retObj
                 }}
@@ -771,10 +828,10 @@ class OpenApiArtGo(OpenApiArtPlugin):
                 return retObj
             }}
 
-            func (obj *{struct}) ToYaml() string {{
+            func (obj *{struct}) ToYaml() (string, error) {{
                 vErr := obj.Validate()
                 if vErr != nil {{
-                    panic(vErr)
+                    return "", vErr
                 }}
                 opts := protojson.MarshalOptions{{
                     UseProtoNames:   true,
@@ -782,12 +839,12 @@ class OpenApiArtGo(OpenApiArtPlugin):
                     EmitUnpopulated: false,
                 }}
                 data, err := opts.Marshal(obj.Msg())
-                if err != nil {{panic(err)}}
+                if err != nil {{return "", err}}
                 data, err = yaml.JSONToYAML(data)
                 if err != nil {{
-                    panic(err)
+                    return "", err
                 }}
-                return string(data)
+                return string(data), nil
             }}
 
             func (obj *{struct}) FromYaml(value string) error {{
@@ -813,10 +870,10 @@ class OpenApiArtGo(OpenApiArtPlugin):
                 return nil
             }}
 
-            func (obj *{struct}) ToJson() string {{
+            func (obj *{struct}) ToJson() (string, error) {{
                 vErr := obj.Validate()
                 if vErr != nil {{
-                    panic(vErr)
+                    return "", vErr
                 }}
                 opts := protojson.MarshalOptions{{
                     UseProtoNames:   true,
@@ -826,9 +883,9 @@ class OpenApiArtGo(OpenApiArtPlugin):
                 }}
                 data, err := opts.Marshal(obj.Msg())
                 if err != nil {{
-                    panic(err)
+                    return "", err
                 }}
-                return string(data)
+                return string(data), nil
             }}
 
             func (obj *{struct}) FromJson(value string) error {{
@@ -876,11 +933,11 @@ class OpenApiArtGo(OpenApiArtPlugin):
 
         interfaces = [
             "// ToPbText marshals {interface} to protobuf text",
-            "ToPbText() string",
+            "ToPbText() (string, error)",
             "// ToYaml marshals {interface} to YAML text",
-            "ToYaml() string",
+            "ToYaml() (string, error)",
             "// ToJson marshals {interface} to JSON text",
-            "ToJson() string",
+            "ToJson() (string, error)",
             "// FromPbText unmarshals {interface} from protobuf text",
             "FromPbText(value string) error",
             "// FromYaml unmarshals {interface} from YAML text",
@@ -924,7 +981,7 @@ class OpenApiArtGo(OpenApiArtPlugin):
         for field in new.interface_fields:
             self._write_field_getter(new, field)
             self._write_field_has(new, field)
-            self._write_field_setter(new, field)
+            self._write_field_setter(new, field, len(internal_items_nil) > 0)
             self._write_field_adder(new, field)
         self._write_validate_method(new)
         self._write_default_method(new)
@@ -934,26 +991,26 @@ class OpenApiArtGo(OpenApiArtPlugin):
             return
         if field.isArray and field.isEnum is False:
             if field.struct:
-                block = ""
+                block = "obj.obj.{name} = []*{pb_pkg_name}.{pb_struct}{{}}".format(
+                    name=field.name, pb_pkg_name=self._protobuf_package_name,
+                    pb_struct=field.external_struct,
+                )
                 if field.setChoiceValue is not None:
                     block = "obj.SetChoice({interface}Choice.{enum})".format(
                         interface=new.interface, enum=field.setChoiceValue
                     )
-                body = """if obj.obj.{name} == nil {{
+                body = """if len(obj.obj.{name}) == 0 {{
                         {block}
-                        obj.obj.{name} = []*{pb_pkg_name}.{pb_struct}{{}}
                     }}
-                    if obj.{internal_name}Holder == nil {{
-                        obj.{internal_name}Holder = new{parent}{interface}Iter().setMsg(obj)
+                    if obj.{internal_name} == nil {{
+                        obj.{internal_name} = new{parent}{interface}Iter().setMsg(obj)
                     }}
-                    return obj.{internal_name}Holder""".format(
+                    return obj.{internal_name}""".format(
                     name=field.name,
-                    pb_pkg_name=self._protobuf_package_name,
-                    pb_struct=field.external_struct,
                     interface=field.external_struct,
                     parent=new.interface,
                     block=block,
-                    internal_name=field.name[0].lower() + field.name[1:]
+                    internal_name=self._get_holder_name(field)
                 )
             else:
                 block = "obj.obj.{name} = make({type}, 0)".format(
@@ -973,7 +1030,6 @@ class OpenApiArtGo(OpenApiArtPlugin):
                     else:
                         block = """
                             obj.SetChoice({interface}Choice.{enum})
-                            obj.obj.{name} = make({type}, 0)
                         """.format(
                             interface=new.interface,
                             enum=field.setChoiceValue,
@@ -990,25 +1046,26 @@ class OpenApiArtGo(OpenApiArtPlugin):
         elif field.struct is not None:
             # at this time proto generation ignores the optional keyword
             # if the type is an object
-            set_enum_choice = None
+            set_choice_or_new = "obj.obj.{name} = New{pb_struct}().Msg()".format(
+                name=field.name,
+                pb_struct=field.external_struct,
+            )
             if field.setChoiceValue is not None:
-                set_enum_choice = """obj.SetChoice({interface}Choice.{enum})""".format(
+                set_choice_or_new = """obj.SetChoice({interface}Choice.{enum})""".format(
                     interface=new.interface,
                     enum=field.setChoiceValue,
                 )
             body = """if obj.obj.{name} == nil {{
-                    {set_enum_choice}
-                    obj.obj.{name} = New{pb_struct}().Msg()
+                    {set_choice_or_new}
                 }}
-                if obj.{internal_name}Holder == nil {{
-                    obj.{internal_name}Holder = &{struct}{{obj: obj.obj.{name}}}
+                if obj.{internal_name} == nil {{
+                    obj.{internal_name} = &{struct}{{obj: obj.obj.{name}}}
                 }}
-                return obj.{internal_name}Holder""".format(
+                return obj.{internal_name}""".format(
                 name=field.name,
-                pb_struct=field.external_struct,
                 struct=field.struct,
-                set_enum_choice=set_enum_choice if set_enum_choice else "",
-                internal_name=field.name[0].lower() + field.name[1:]
+                set_choice_or_new=set_choice_or_new,
+                internal_name=self._get_holder_name(field)
             )
         elif field.isEnum:
             enum_types = []
@@ -1074,25 +1131,13 @@ class OpenApiArtGo(OpenApiArtPlugin):
             return
         elif field.isPointer:
             set_enum_choice = None
-            default = None
             if field.setChoiceValue is not None:
-                if field.default is not None:
-                    if field.type == "string":
-                        value = '"{}"'.format(field.default)
-                    elif field.type in ["int32", "int64", "float32", "float64"]:
-                        value = "{fieldtype}({value})".format(fieldtype=field.type, value=field.default)
-                    else:
-                        value = field.default
-                    default = """value := {value}
-                            obj.obj.{fieldname} = &value""".format(value=value, fieldname=field.name)
                 set_enum_choice = """
                     if obj.obj.{fieldname} == nil {{
                         obj.SetChoice({interface}Choice.{enum})
-                        {default}
                     }}
                 """.format(
                     fieldname=field.name,
-                    default=default if default else "",
                     interface=new.interface, enum=field.setChoiceValue
                 )
             body = """{set_enum_choice}
@@ -1103,22 +1148,14 @@ class OpenApiArtGo(OpenApiArtPlugin):
             )
         else:
             set_enum_choice = None
-            default = None
             if field.setChoiceValue is not None:
-                if field.default is not None:
-                    default = "obj.obj.{fieldname} = {value}".format(
-                        value='"{}"'.format(field.default) if "string" in field.type else field.default,
-                        fieldname=field.name
-                    )
                 set_enum_choice = """
                     if obj.obj.{fieldname} == nil {{
                         obj.SetChoice({interface}Choice.{enum})
-                        {default}
                     }}
                 """.format(
                     fieldname=field.name,
                     interface=new.interface, enum=field.setChoiceValue,
-                    default=default if default else ""
                 )
             body = """{set_enum_choice}\n return obj.obj.{fieldname}""".format(
                 fieldname=field.name,
@@ -1140,7 +1177,7 @@ class OpenApiArtGo(OpenApiArtPlugin):
             )
         )
 
-    def _write_field_setter(self, new, field):
+    def _write_field_setter(self, new, field, set_nil):
         if field.setter_method is None:
             return
 
@@ -1179,10 +1216,10 @@ class OpenApiArtGo(OpenApiArtPlugin):
                     interface=new.interface,
                     fieldname=field.name,
                 )
-            enum_set = [self._get_external_field_name(name) for name in field.enums]
+            enum_set = {self._get_external_field_name(name) : name for name in field.enums}
             enum_body = []
             for enum_field in new.interface_fields:
-                if enum_field.name not in enum_set:
+                if enum_set.get(enum_field.name) is None:
                     continue
                 if enum_field.isEnum:
                     enum_body.append(
@@ -1192,7 +1229,57 @@ class OpenApiArtGo(OpenApiArtPlugin):
                         )
                     )
                     continue
-                enum_body.append("obj.obj.{name} = nil".format(name=enum_field.name))
+                if enum_field.struct is not None and enum_field.isArray is False:
+                    enum_body.append("""
+                        if value == {interface}{name}.{enumupper} {{
+                            obj.obj.{enumname} = New{struct}().Msg()
+                        }}
+                    """.format(
+                        interface=new.interface, name=field.name, enumname=enum_field.name,
+                        struct=self._get_external_struct_name(enum_field.struct),
+                        enumupper=enum_set.get(enum_field.name).upper()
+                    ))
+                elif enum_field.struct is not None and enum_field.isArray:
+                    enum_body.append("""
+                        if value == {interface}{name}.{enumupper} {{
+                            obj.obj.{enumname} = []*{pb_pkg}.{struct}{{}}
+                        }}
+                    """.format(
+                        interface=new.interface, name=field.name, enumname=enum_field.name,
+                        struct=self._get_external_struct_name(enum_field.struct),
+                        enumupper=enum_set.get(enum_field.name).upper(),
+                        pb_pkg=self._protobuf_package_name
+                    ))
+                elif enum_field.default is not None:
+                    default_value = "" if enum_field.default is None else enum_field.default
+                    if enum_field.isArray:
+
+                        default_value = '{type}{{{value}}}'.format(
+                            type=enum_field.type,
+                            value='"{}"'.format('", "'.join(default_value))
+                            if "string" in enum_field.type else ",".join(
+                                [str(e) for e in default_value])
+                        )
+                    else:
+                        default_value = '"{}"'.format(default_value) \
+                            if enum_field.type == "string" \
+                            else "{}({})".format(enum_field.type, default_value)
+                    enum_body.append("""
+                        if value == {interface}{name}.{enumupper} {{
+                            defaultValue := {default_value}
+                            obj.obj.{enumname} = {point}defaultValue
+                        }}
+                    """.format(
+                        interface=new.interface, name=field.name, enumname=enum_field.name,
+                        default_value=default_value,
+                        enumupper=enum_set.get(enum_field.name).upper(),
+                        point="" if enum_field.isArray else "&"
+                    ))
+                enum_body.insert(0, "obj.obj.{name} = nil".format(name=enum_field.name))
+                if enum_field.struct is not None:
+                    enum_body.insert(1, "obj.{name} = nil".format(
+                        name=self._get_holder_name(enum_field)))
+
             self._write(
                 """func (obj* {struct}) Set{fieldname}(value {interface}{fieldname}Enum) {interface} {{
                 intValue, ok := {pb_pkg_name}.{interface}_{fieldname}_Enum_value[string(value)]
@@ -1216,7 +1303,12 @@ class OpenApiArtGo(OpenApiArtPlugin):
             )
             return
         elif field.struct is not None:
-            body = """obj.{fieldname}().SetMsg(value.Msg())""".format(
+            body = """{set_nil} = nil
+            obj.obj.{name} = value.Msg()
+            """.format(
+                set_nil="obj.{}".format(self._get_holder_name(field))
+                if set_nil is True else "",
+                name=field.name,
                 fieldname=self._get_external_struct_name(field.name),
             )
         elif field.isPointer:
@@ -1345,7 +1437,7 @@ class OpenApiArtGo(OpenApiArtPlugin):
                 field_name=field.name,
                 pb_pkg_name=self._protobuf_package_name,
                 field_type=field.type,
-                internal_items_name="{}Slice".format(field.struct)
+                internal_items_name=self._get_holder_name(field, True)
             )
         )
 
@@ -1871,6 +1963,7 @@ class OpenApiArtGo(OpenApiArtPlugin):
                         value='"{0}"'.format(field.default) if field.type == "string" else field.default,
                     )
         if choice_body is not None:
+            enum_fields = []
             body = choice_body.replace("<choice_fields>", "\n".join(enum_fields) if enum_fields != [] else "") + body
 
         self._write(
@@ -1942,6 +2035,11 @@ class OpenApiArtGo(OpenApiArtPlugin):
                 description += "// {line}\n".format(line=line.strip())
         return description.strip("\n")
 
+    def _get_holder_name(self, field, isIter=False):
+        if isIter:
+            return "{}Slice".format(field.struct)
+        return "{}Holder".format(field.name[0].lower() + field.name[1:])
+
     def _format_go_file(self):
         """Format the generated go code"""
         try:
