@@ -33,6 +33,8 @@ def test_serialize_deserialize(api):
     config.a = "asdf"
     config.b = 1.1
     config.c = 1
+    config.required_object.e_a = 1.1
+    config.required_object.e_b = 1.2
     config.d_values = [config.A, config.B, config.C]
     config.level.l1_p1.l2_p1.l3_p1 = "test"
     config.level.l1_p2.l4_p1.l1_p2.l4_p1.l1_p1.l2_p1.l3_p1 = "test"
